package com.bocloud.cmp.provider;

import com.bocloud.cmp.custom.CustomVpcClient;
import com.bocloud.cmp.tencentCfs.v20190719.CfsClient;
import org.apache.commons.lang3.StringUtils;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.tencentcloudapi.billing.v20180709.BillingClient;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.clb.v20180317.ClbClient;
import com.tencentcloudapi.common.AbstractClient;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesResponse;
import com.tencentcloudapi.vpc.v20170312.VpcClient;

/**
 * 腾讯云
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
public class TencentProvider extends GenericProvider implements SuperProvider {

    private String secretId;
    private String secretKey;
    private Credential credential;
    private String endpoint; // 平台地址 api3.401.tcecqpoc.fsphere.cn
    private Butler butler;
    private String projectId = "0"; // 默认项目
    private String regionId = "ap-beijing"; // 选择大区
    private ClientProfile clientProfile = new ClientProfile(); // 接入地域
    private HttpProfile httpProfile = new HttpProfile();

    private static final String TENCENT_PUBLIC_URL = ".tencentcloudapi.com";
    private static final int CONN_TIMEOUT = 60 * 10;
    public static final int REQUEST_LIMIT = 50;

    public TencentProvider(Butler butler) {
        super(butler);
        this.secretId = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.regionId = butler.getRegionId();
        this.credential = new Credential(secretId, secretKey);
        this.endpoint = butler.getUrl();
        this.butler = butler;
    }

    private void configuration(TencentEnum type) {
        httpProfile.setConnTimeout(CONN_TIMEOUT);
        clientProfile.setHttpProfile(httpProfile);
        if (StringUtils.isBlank(regionId)) {
            this.regionId = "ap-beijing";
        }
        httpProfile.setEndpoint(type.toString().toLowerCase() + "." + regionId + TENCENT_PUBLIC_URL);
    }

    public AbstractClient getClient(TencentEnum type) {
        AbstractClient abstractClient = null;
        this.configuration(type);
        switch (type) {
        case CVM:
            abstractClient = new CvmClient(credential, regionId, clientProfile);
            break;
        case CBS:
            abstractClient = new CbsClient(credential, regionId, clientProfile);
            break;
        case VPC:
            abstractClient = new CustomVpcClient(credential, regionId, clientProfile);
            break;
        case CLB:
            abstractClient = new ClbClient(credential, regionId, clientProfile);
            break;
        case BILLING:
            abstractClient = new BillingClient(credential, regionId, clientProfile);
            break;
        case CFS:
            abstractClient = new CfsClient(credential,regionId,clientProfile);
            break;
        default:
            break;
        }
        return abstractClient;
    }

    @Override
    public boolean accessable() {
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            DescribeInstancesRequest req = new DescribeInstancesRequest();
            DescribeInstancesResponse response = client.DescribeInstances(req);
            response.getInstanceSet();
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public String getSecretId() {
        return secretId;
    }

    public void setSecretId(String secretId) {
        this.secretId = secretId;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    public Credential getCredential() {
        return credential;
    }

    public void setCredential(Credential credential) {
        this.credential = credential;
    }

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public Butler getButler() {
        return butler;
    }

    public void setButler(Butler butler) {
        this.butler = butler;
    }

    public String getProjectId() {
        return projectId;
    }

    public void setProjectId(String projectId) {
        this.projectId = projectId;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public ClientProfile getClientProfile() {
        return clientProfile;
    }

    public void setClientProfile(ClientProfile clientProfile) {
        this.clientProfile = clientProfile;
    }

    public HttpProfile getHttpProfile() {
        return httpProfile;
    }

    public void setHttpProfile(HttpProfile httpProfile) {
        this.httpProfile = httpProfile;
    }

}