package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.tencentconvertor.BillingConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.billing.v20180709.BillingClient;
import com.tencentcloudapi.billing.v20180709.models.BillDetail;
import com.tencentcloudapi.billing.v20180709.models.BillResourceSummary;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillDetailRequest;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillDetailResponse;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillResourceSummaryRequest;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillResourceSummaryResponse;

/**
 * 腾讯云计费相关
 * 
 * @author ymh
 * @时间 2019年4月2日
 */
@Provider("bocloud.cmp.tencent")
public class TencentBillingProvider extends TencentProvider implements BillingProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentBillingProvider.class);

	private BillingConvertor convertor = new BillingConvertor();

	private static final int PAGE_SIZE = 100;

	public TencentBillingProvider(Butler butler) {
		super(butler);
	}

	/**
	 * 账单明细数据
	 */
	@Override
	public BocloudResult billDetailList(String month) {
		try {
			DescribeBillResourceSummaryRequest req = new DescribeBillResourceSummaryRequest();
			req.setPeriodType("byPayTime");
			req.setMonth(month);
			BillingClient billClient = (BillingClient) this.getClient(TencentEnum.BILLING);
			Integer offset = 0;
			List<BillResourceSummary> summarys = new ArrayList<>();
			while (offset >= 0) {
				req.setOffset(offset * PAGE_SIZE);
				req.setLimit(PAGE_SIZE);
				DescribeBillResourceSummaryResponse resp = billClient.DescribeBillResourceSummary(req);
				BillResourceSummary[] resourceSummarySet = resp.getResourceSummarySet();
				if (null != resourceSummarySet && resourceSummarySet.length > 0) {
					summarys.addAll(Arrays.asList(resourceSummarySet));
					offset += 1;
				} else {
					offset = -1;
				}
			}
			return new BocloudResult(true, convertor.convertList(summarys), "查询成功！");
		} catch (Exception e) {
			logger.error("查询失败 ： " + e.getMessage());
			return new BocloudResult(false, "查询失败 ！" + e.getMessage());
		}
	}

	/**
	 * 账单明细资源汇总数据
	 */
	public BocloudResult billResourceSummaryList(String month) {
		try {
			DescribeBillDetailRequest req = new DescribeBillDetailRequest();
			req.setPeriodType("byPayTime");
			req.setMonth(month);
			List<BillDetail> list = new ArrayList<>();
			Integer offset = 0;
			BillingClient billClient = (BillingClient) this.getClient(TencentEnum.BILLING);
			while (offset >= 0) {
				req.setOffset(offset * PAGE_SIZE);
				req.setLimit(PAGE_SIZE);
				DescribeBillDetailResponse resp = billClient.DescribeBillDetail(req);
				BillDetail[] detailSet = resp.getDetailSet();
				if (null != detailSet && detailSet.length > 0) {
					list.addAll(Arrays.asList(detailSet));
					offset += 1;
				} else {
					offset = -1;
				}
			}
			return new BocloudResult(true, list, "查询成功");
		} catch (Exception e) {
			logger.error("查询订单Error : " + e.getMessage());
			return new BocloudResult(false, "查询订单Error : " + e.getMessage());
		}
	}
}
