package com.bocloud.cmp.provider.common;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.FlavorConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZoneInstanceConfigInfosRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZoneInstanceConfigInfosResponse;
import com.tencentcloudapi.cvm.v20170312.models.Filter;
import com.tencentcloudapi.cvm.v20170312.models.InstanceTypeQuotaItem;

/**
 * 腾讯云规格
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentFlavorProvider extends TencentProvider implements FlavorProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentFlavorProvider.class);

    private BeanConvertor<FlavorModel, InstanceTypeQuotaItem> convertor = new FlavorConvertor();

    public TencentFlavorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        return publicSync();
    }

    private BocloudResult publicSync() {
        BocloudResult result = new BocloudResult(false, "");
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            DescribeZoneInstanceConfigInfosRequest req = new DescribeZoneInstanceConfigInfosRequest();
            Filter f1 = new Filter();
            f1.setName("instance-charge-type");
            // PREPAID：表示预付费，即包年包月 | POSTPAID_BY_HOUR：表示后付费，即按量计费
            f1.setValues(new String[] { /*"PREPAID"*/ "POSTPAID_BY_HOUR" });
            Filter[] filters = new Filter[] { f1 };
            req.setFilters(filters);
            DescribeZoneInstanceConfigInfosResponse resp = client.DescribeZoneInstanceConfigInfos(req);
            InstanceTypeQuotaItem[] instanceTypeQuotaSet = resp.getInstanceTypeQuotaSet();
            result = new BocloudResult(true, convertor.convertList(Arrays.asList(instanceTypeQuotaSet)), "查询成功");
        } catch (TencentCloudSDKException e) {
            logger.error("规格查询失败:", e);
            result = new BocloudResult(false, e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        return null;
    }

    @Override
    public BocloudResult remove(String flavorId) {
        return null;
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        return null;
    }

}
