package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.model.tencent.storage.FileSystemModel;
import com.bocloud.cmp.provider.storage.TencentFileSystemsProvider;
import com.bocloud.cmp.provider.tencent.storage.FileSystemProvider;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.compute.ImageProvider;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.cmp.provider.compute.SnapshotProvider;
import com.bocloud.cmp.provider.compute.TencentImageProvider;
import com.bocloud.cmp.provider.compute.TencentServerProvider;
import com.bocloud.cmp.provider.compute.TencentSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.security.TencentKeypairProvider;
import com.bocloud.cmp.provider.storage.TencentVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.cmp.tencentconvertor.RegionConvertor;
import com.bocloud.cmp.tencentconvertor.ZoneConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeRegionsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeRegionsResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZonesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZonesResponse;
import com.tencentcloudapi.cvm.v20170312.models.RegionInfo;
import com.tencentcloudapi.cvm.v20170312.models.ZoneInfo;

/**
 * Tencent同步数据组装类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentLocationProvider extends TencentProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentLocationProvider.class);

    private Butler butler;

    public TencentLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    private BeanConvertor<RegionModel, RegionInfo> regionConvertor = new RegionConvertor();
    private BeanConvertor<ZoneModel, ZoneInfo> zoneConvertor = new ZoneConvertor();

    @Override
    public BocloudResult sync() {
        try {
            TencentSyncModel syncModel = new TencentSyncModel();
            logger.info("同步地域信息...start..." + this.getRegionId());
            List<RegionModel> regions = new ArrayList<>();
            BocloudResult listRegions = this.listRegions();
            if(listRegions.isFailed()) {
                return listRegions;
            }
            RegionModel regionModel = new RegionModel();
            List<RegionModel> regionList = (List<RegionModel>)listRegions.getData();
            for (RegionModel regModel : regionList) {
                if(this.getRegionId().equalsIgnoreCase(regModel.getRegionId())) {
                    regionModel = regModel;
                    break;
                }
            }
            if (!StringUtils.isEmpty(this.getRegionId())) {
                BocloudResult listZones = this.listZones(regionModel);
                regionModel
                    .setZones(JSONObject.parseArray(JSONObject.toJSONString(listZones.getData()), ZoneModel.class));
            }
            regions.add(regionModel);
            syncModel.setRegionModels(regions);
            logger.info("同步地域信息...end...");
            logger.info("同步规格Flavor...start...");
            syncFlavor(syncModel);
            logger.info("同步规格Flavor...end...");
            logger.info("同步镜像Image...start...");
            syncImage(syncModel);
            logger.info("同步镜像Image...end...");
            logger.info("同步卷Disk...start...");
            syncVolume(syncModel);
            logger.info("同步卷Disk...end...");
            logger.info("同步快照Snapshot...start...");
            syncSnapshot(syncModel);
            logger.info("同步快照Snapshot...end...");
            logger.info("同步安全组SecurityGroup...start...");
            syncSecurityGroup(syncModel);
            logger.info("同步安全组SecurityGroup...end...");
            logger.info("同步虚拟机Server...start...");
            syncServer(syncModel);
            logger.info("同步虚拟机Server...end...");
            logger.info("同步网络VPC...start...");
            syncVpc(syncModel);
            logger.info("同步网络VPC...end...");
            logger.info("同步路由表RouteTable...start...");
            syncTableRoute(syncModel);
            logger.info("同步路由表RouteTable...end...");
            logger.info("同步keypair...start...");
            syncKeypair(syncModel);
            logger.info("同步keypair...end...");
            logger.info("同步子网subnet...start...");
            syncSubnet(syncModel);
            logger.info("同步子网subnet...end...");
            logger.info("ip....start");
            syncFloatingIp(syncModel);
            logger.info("ip....end");
            logger.info("networkcard 网卡.... start");
            syncNetworkCard(syncModel);
            logger.info("networkcard....end");
            logger.info("同步文件存储subfile...start...");
            syncFileSystems(syncModel);
            logger.info("同步文件存储file...End");
            return new BocloudResult(true, syncModel, "Tencent 同步成功！");
        } catch (Exception e) {
            logger.error("腾讯云同步失败： " + e);
            return new BocloudResult(false, e.getMessage(), "腾讯云同步失败");
        }
    }

    private void syncNetworkCard(TencentSyncModel syncModel) {
        NetworkCardProvider networkCardProvider = new TencentNetworkCardProvider(butler);
        BocloudResult networkResult = networkCardProvider.list();
        if (networkResult.isSuccess()) {
            List<NetworkCardModel> networkCardModels =
                JSONObject.parseArray(JSONObject.toJSONString(networkResult.getData()), NetworkCardModel.class);
            syncModel.setNetworkCardModels(networkCardModels);
        }
    }

    private void syncFloatingIp(TencentSyncModel syncModel) {
        FloatingIPProvider floatingIpProvider = new TencentFloatingIpProvider(butler);
        BocloudResult ipResult = floatingIpProvider.list();
        if (ipResult.isSuccess()) {
            List<FloatingIpModel> parseArray =
                JSONObject.parseArray(JSONObject.toJSONString(ipResult.getData()), FloatingIpModel.class);
            syncModel.setFloatModels(parseArray);
        }
    }

    private void syncSubnet(TencentSyncModel syncModel) {
        SubnetProvider subnetProvider = new TencentSubnetProvider(butler);
        BocloudResult subnetResult = subnetProvider.list();
        if (subnetResult.isSuccess()) {
            List<SubnetModel> subnetModels =
                JSONObject.parseArray(JSONObject.toJSONString(subnetResult.getData()), SubnetModel.class);
            for (SubnetModel subnetModel : subnetModels) {
                subnetModel.setRegionId(this.getRegionId());
            }
            syncModel.setSubnetModels(subnetModels);
        }
    }

    private void syncKeypair(TencentSyncModel syncModel) {
        KeypairProvider keypairProvider = new TencentKeypairProvider(butler);
        BocloudResult keypairResult = keypairProvider.list();
        if (keypairResult.isSuccess()) {
            List<KeypairModel> keypairModels =
                JSONObject.parseArray(JSONObject.toJSONString(keypairResult.getData()), KeypairModel.class);
            syncModel.setKeypairModels(keypairModels);
        }
    }

    private void syncVpc(TencentSyncModel syncModel) {
        VpcProvider vpcProvider = new TencentVpcProvider(butler);
        BocloudResult vpcResult = vpcProvider.list();
        if (vpcResult.isSuccess()) {
            List<VpcModel> vpcModels =
                JSONObject.parseArray(JSONObject.toJSONString(vpcResult.getData()), VpcModel.class);
            for (VpcModel vpcModel : vpcModels) {
                vpcModel.setRegionId(this.getRegionId());
            }
            syncModel.setVpcModels(vpcModels);
        }
    }

    private void syncTableRoute(TencentSyncModel syncModel) {
        TencentRouteTableProvider routeTableProvider = new TencentRouteTableProvider(butler);
        BocloudResult routeTableResult = routeTableProvider.list();
        if (routeTableResult.isSuccess()) {
            List<RouteTableModel> routeTableModels =
                    JSONObject.parseArray(JSONObject.toJSONString(routeTableResult.getData()), RouteTableModel.class);
            for (RouteTableModel routeTableModel : routeTableModels) {
                routeTableModel.setRegionId(this.getRegionId());
            }
            syncModel.setRouteTableModels(routeTableModels);
        }
    }

    private void syncServer(TencentSyncModel syncModel) {
        ServerProvider serverProvider = new TencentServerProvider(butler);
        BocloudResult serverResult = serverProvider.list();
        if (serverResult.isSuccess()) {
            List<ServerModel> serverModels =
                JSONObject.parseArray(JSONObject.toJSONString(serverResult.getData()), ServerModel.class);
            for (ServerModel serverModel : serverModels) {
                serverModel.setRegionId(this.getRegionId());
            }
            syncModel.setServerModels(serverModels);
        }
    }

    private void syncSecurityGroup(TencentSyncModel syncModel) {
        SecurityGroupProvider secgProvider = new TencentSecuGroupProvider(butler);
        TencentSecuGroupRuleProvider securityGroupRuleProvider = new TencentSecuGroupRuleProvider(butler);
        BocloudResult securityResult = secgProvider.list();
        if (securityResult.isSuccess()) {
            List<SecurityGroupModel> securityGroupModels =
                JSONObject.parseArray(JSONObject.toJSONString(securityResult.getData()), SecurityGroupModel.class);
            for (SecurityGroupModel securityGroupModel : securityGroupModels) {
                securityGroupModel.setRegionId(this.getRegionId());
                // 同步安全组下对应规则
                BocloudResult secRuleResult = securityGroupRuleProvider.list(securityGroupModel.getGroupUuid());
                if (secRuleResult.isSuccess()) {
                    List<SecurityGroupRuleModel> parseArray = JSONObject
                        .parseArray(JSONObject.toJSONString(secRuleResult.getData()), SecurityGroupRuleModel.class);
                    securityGroupModel.setModels(parseArray);
                }
            }
            syncModel.setSecurityGroupModels(securityGroupModels);
        }
    }

    private void syncSnapshot(TencentSyncModel syncModel) {
        SnapshotProvider snapshotProvider = new TencentSnapshotProvider(butler);
        BocloudResult snapshotResult = snapshotProvider.list();
        if (snapshotResult.isSuccess()) {
            List<SnapshotModel> snapshotModels =
                JSONObject.parseArray(JSONObject.toJSONString(snapshotResult.getData()), SnapshotModel.class);
            for (SnapshotModel snapshotModel : snapshotModels) {
                snapshotModel.setRegionId(this.getRegionId());
            }
            syncModel.setSnapshotModels(snapshotModels);
        }
    }

    private void syncVolume(TencentSyncModel syncModel) {
        VolumeProvider volumeProvider = new TencentVolumeProvider(butler);
        BocloudResult volumeResult = volumeProvider.list();
        if (volumeResult.isSuccess()) {
            List<VolumeModel> volumeModels =
                JSONObject.parseArray(JSONObject.toJSONString(volumeResult.getData()), VolumeModel.class);
            for (VolumeModel volumeModel : volumeModels) {
                volumeModel.setRegionId(this.getRegionId());
            }
            syncModel.setVolumeModels(volumeModels);
        }
    }

    private void syncImage(TencentSyncModel syncModel) {
        ImageProvider imageProvider = new TencentImageProvider(butler);
        BocloudResult imageResult = imageProvider.list();
        if (imageResult.isSuccess()) {
            List<ImageModel> imageModels =
                JSONObject.parseArray(JSONObject.toJSONString(imageResult.getData()), ImageModel.class);
            for (ImageModel imageModel : imageModels) {
                imageModel.setRegionId(this.getRegionId());
            }
            syncModel.setImageModels(imageModels);
        }
    }

    private void syncFileSystems(TencentSyncModel syncModel){
        FileSystemProvider fileSystemProvider = new TencentFileSystemsProvider(butler);
        BocloudResult fileSystemResult = fileSystemProvider.list();
        if (fileSystemResult.isSuccess()){
            List<FileSystemModel> fileSystemModels =
                    JSONObject.parseArray(JSONObject.toJSONString(fileSystemResult.getData()),FileSystemModel.class);
            for (FileSystemModel fileSystemModel : fileSystemModels){
                fileSystemModel.setRegionId(this.getRegionId());
            }
            syncModel.setFileSystemModels(fileSystemModels);
        }

    }

    private void syncFlavor(TencentSyncModel syncModel) {
        FlavorProvider flavorProvider = new TencentFlavorProvider(butler);
        BocloudResult flavorResult = flavorProvider.sync();
        if (flavorResult.isSuccess()) {
            List<FlavorModel> flavorArray =
                JSONObject.parseArray(JSONObject.toJSONString(flavorResult.getData()), FlavorModel.class);
            for (FlavorModel flavorModel : flavorArray) {
                flavorModel.setRegionId(this.getRegionId());
            }
            syncModel.setFlavorModels(flavorArray);
        }
    }

    @Override
    public BocloudResult listRegions() {
        try {
            DescribeRegionsRequest req = DescribeRegionsRequest.fromJsonString("{}", DescribeRegionsRequest.class);
            CvmClient client = (CvmClient)this.getClient(TencentEnum.CVM);
            DescribeRegionsResponse resp = client.DescribeRegions(req);
            RegionInfo[] regionSet = resp.getRegionSet();
            return new BocloudResult(true, regionConvertor.convertList(Arrays.asList(regionSet)), "查询成功");
        } catch (Exception e) {
            logger.error("查询失败： " + e);
            return new BocloudResult(false, e.getMessage(), "查询失败");
        }
    }

    /**
     * 汉化腾讯云区域
     * 
     * @param model
     * @return
     */
    public BocloudResult listZones(RegionModel model) {
        try {
            DescribeZonesRequest req = new DescribeZonesRequest();
            CvmClient client = (CvmClient)this.getClient(TencentEnum.CVM);
            client.setRegion(model.getRegionId());
            DescribeZonesResponse resp = client.DescribeZones(req);
            ZoneInfo[] zoneSet = resp.getZoneSet();
            List<ZoneModel> convertList = zoneConvertor.convertList(Arrays.asList(zoneSet));
            if (!ListTool.isEmpty(convertList)) {
                String regionName = model.getName();
                if (!StringUtils.isEmpty(regionName)) {
                    regionName = regionName.substring(regionName.indexOf("(") + 1, regionName.indexOf(")"));
                }
                for (ZoneModel zoneModel : convertList) {
                    String zoneName = zoneModel.getName();
                    zoneModel.setName(this.getZoneName(zoneName, regionName));
                }
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("查询失败 ： " + e);
            return new BocloudResult(false, e.getMessage(), "查询失败");
        }
    }

    private String getZoneName(String zoneName, String regionName) {
        String newZoneName = zoneName;
        if (zoneName.endsWith("a") || zoneName.endsWith("1")) {
            newZoneName = regionName + "一区";
        }
        if (zoneName.endsWith("b") || zoneName.endsWith("2")) {
            newZoneName = regionName + "二区";
        }
        if (zoneName.endsWith("c") || zoneName.endsWith("3")) {
            newZoneName = regionName + "三区";
        }
        if (zoneName.endsWith("d") || zoneName.endsWith("4")) {
            newZoneName = regionName + "四区";
        }
        if (zoneName.endsWith("e") || zoneName.endsWith("5")) {
            newZoneName = regionName + "五区";
        }
        if (zoneName.endsWith("f") || zoneName.endsWith("6")) {
            newZoneName = regionName + "六区";
        }
        if (zoneName.endsWith("g") || zoneName.endsWith("7")) {
            newZoneName = regionName + "七区";
        }
        if (zoneName.endsWith("g") || zoneName.endsWith("7")) {
            newZoneName = regionName + "七区";
        }
        if (zoneName.endsWith("h") || zoneName.endsWith("8")) {
            newZoneName = regionName + "八区";
        }
        return newZoneName;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel arg0) {
        return null;
    }

}
