package com.bocloud.cmp.provider.compute;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.TencentNetworkCardProvider;
import com.bocloud.cmp.provider.storage.TencentVolumeProvider;
import com.bocloud.cmp.tencentconvertor.ServerConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.AssociateInstancesKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DataDisk;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusResponse;
import com.tencentcloudapi.cvm.v20170312.models.DisassociateInstancesKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DisassociateInstancesKeyPairsResponse;
import com.tencentcloudapi.cvm.v20170312.models.Instance;
import com.tencentcloudapi.cvm.v20170312.models.InstanceChargePrepaid;
import com.tencentcloudapi.cvm.v20170312.models.InstanceStatus;
import com.tencentcloudapi.cvm.v20170312.models.InternetAccessible;
import com.tencentcloudapi.cvm.v20170312.models.LoginSettings;
import com.tencentcloudapi.cvm.v20170312.models.ModifyInstancesAttributeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ModifyInstancesAttributeResponse;
import com.tencentcloudapi.cvm.v20170312.models.Placement;
import com.tencentcloudapi.cvm.v20170312.models.RebootInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstanceRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesInternetMaxBandwidthRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesInternetMaxBandwidthResponse;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesPasswordRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesPasswordResponse;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesTypeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResizeInstanceDisksRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResizeInstanceDisksResponse;
import com.tencentcloudapi.cvm.v20170312.models.RunInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.RunInstancesResponse;
import com.tencentcloudapi.cvm.v20170312.models.StartInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.StopInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.SystemDisk;
import com.tencentcloudapi.cvm.v20170312.models.TerminateInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.TerminateInstancesResponse;
import com.tencentcloudapi.cvm.v20170312.models.VirtualPrivateCloud;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.AssociateAddressRequest;
import com.tencentcloudapi.vpc.v20170312.models.AssociateAddressResponse;
import com.tencentcloudapi.vpc.v20170312.models.DisassociateAddressRequest;
import com.tencentcloudapi.vpc.v20170312.models.DisassociateAddressResponse;

/**
 * tencent虚拟机操作类
 *
 * @ Description： @ Author ：丁明威 @ Date ：Created in 16:36 2018/11/14 @ Modified
 * By：
 */
@Provider("bocloud.cmp.tencent")
public class TencentServerProvider extends TencentProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentServerProvider.class);

	private BeanConvertor<ServerModel, Instance> convertor = new ServerConvertor();

	private static final String[] INSTANCES_STAUTS = { "RUNNING", "STOPED", "STOPPED" };

	private Butler butler;

	public TencentServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		Integer offset = 0;
		DescribeInstancesRequest req = new DescribeInstancesRequest();
		List<Instance> instances = new ArrayList<>();
		try {
			req.setLimit(REQUEST_LIMIT);
			DescribeInstancesResponse response = ((CvmClient) this.getClient(TencentEnum.CVM)).DescribeInstances(req);
			instances.addAll(Arrays.asList(response.getInstanceSet()));
			// 循环分页查询
			while (instances.size() >= REQUEST_LIMIT + offset) {
				offset = offset + REQUEST_LIMIT;
				req.setOffset(offset);
				response = ((CvmClient) this.getClient(TencentEnum.CVM)).DescribeInstances(req);
				instances.addAll(Arrays.asList(response.getInstanceSet()));
			}
			List<ServerModel> convertList = convertor.convertList(instances);

			List<String> instanceIdList = new ArrayList<>();
			for (ServerModel serverModel : convertList) {
				instanceIdList.add(serverModel.getInstanceId());
				serverModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, convertList, "success");
		} catch (Exception e) {
			logger.error("list cloud server error:", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult getInstances(String instanceId) {
		BocloudResult result = new BocloudResult(false, "查询失败！");
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			DescribeInstancesStatusRequest req = new DescribeInstancesStatusRequest();
			req.setInstanceIds(new String[] { instanceId });
			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			// 5分钟
			while (endTime - startTime < 20 * 60 * 1000) {
				Thread.sleep(6000);
				DescribeInstancesStatusResponse resp = client.DescribeInstancesStatus(req);
				InstanceStatus[] instanceStatusSet = resp.getInstanceStatusSet();
				if (instanceStatusSet == null || instanceStatusSet.length <= 0) {
					break;
				}
				InstanceStatus instanceStatus = instanceStatusSet[0];
				if (Arrays.asList(INSTANCES_STAUTS)
						.contains(Optional.ofNullable(instanceStatus.getInstanceState()).orElse("").toUpperCase())) {
					result = this.detail(instanceId);
					break;
				}
				endTime = System.currentTimeMillis();
			}
		} catch (Exception e) {
			logger.error("查询虚拟机状态失败： " + e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult create(ServerModel server) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			if (null != server.getRegionId() && !"".equals(server.getRegionId())) {
				client.setRegion(server.getRegionId());
			}
			RunInstancesRequest req = new RunInstancesRequest();
			this.instanceChargeTypeParams(server, req);
			// 地区 项目
			this.projectParams(server, req);
			// 名称
			req.setInstanceName(server.getName());
			req.setImageId(server.getImageUuid());
			if (null != server.getName() && !"".equals(server.getName())) {
				req.setInstanceName(server.getName());
			}
			if (null != server.getFlavorUuid() && !"".equals(server.getFlavorUuid())) {
				req.setInstanceType(server.getFlavorUuid()); // 规格
			}
			Integer count = server.getCount();
			if (null == count || count <= 0) {
				count = 1;
			}
			req.setInstanceCount(count);
			this.systemDiskParams(server, req);
			// volume
			this.volumeParams(server, req);
			// vpc
			this.vpcParams(server, req);
			// 公网
//            this.publicIp(server, req);
			// 登录方式
			this.loginParams(server, req);
			List<String> securityGroups = server.getSecurityGroups();
			if (null != securityGroups && !securityGroups.isEmpty()) {
				String[] securityGroupIds = new String[securityGroups.size()];
				req.setSecurityGroupIds(securityGroups.toArray(securityGroupIds));
			}
			RunInstancesResponse resp = client.RunInstances(req);
			String[] instanceIdSet = resp.getInstanceIdSet();
			return getInstances(instanceIdSet[0]);
		} catch (Exception e) {
			logger.error("创建实例失败： " + e);
			return new BocloudResult(false, "创建实例失败", e.getMessage());
		}
	}

	private void projectParams(ServerModel server, RunInstancesRequest req) {
		Placement placement = new Placement();
		placement.setZone(server.getZone());
		String proId = this.getProjectId();
		if (null != proId && !"".equals(proId)) {
			placement.setProjectId(Integer.parseInt(this.getProjectId()));
		}
		req.setPlacement(placement);
	}

	private void instanceChargeTypeParams(ServerModel server, RunInstancesRequest req) {
		// 付费方式
		// PREPAID：预付费，即包年包月
		// POSTPAID_BY_HOUR：按小时后付费
		String instanceChargeType = server.getInstanceChargeType();
		if (null == instanceChargeType || "".equals(instanceChargeType)) {
			instanceChargeType = "POSTPAID_BY_HOUR";
		}
		req.setInstanceChargeType(instanceChargeType);
		// 购买时长
		if ("PREPAID".equals(instanceChargeType)) {
			InstanceChargePrepaid instanceChargePrepaid = new InstanceChargePrepaid();
			instanceChargePrepaid.setPeriod(server.getInstancePeriod());
			/**
			 * •NOTIFY_AND_AUTO_RENEW：通知过期且自动续费 •NOTIFY_AND_MANUAL_RENEW：通知过期不自动续费
			 * •DISABLE_NOTIFY_AND_MANUAL_RENEW：不通知过期不自动续费
			 * 默认取值：NOTIFY_AND_MANUAL_RENEW。若该参数指定为NOTIFY_AND_AUTO_RENEW，在账户余额充足的情况下，实例到期后将按月自动续费。
			 */
			if (null != server.getInstanceRenewFlag()) {
				instanceChargePrepaid.setRenewFlag(server.getInstanceRenewFlag());
			}
			req.setInstanceChargePrepaid(instanceChargePrepaid);
		}
	}

	private void vpcParams(ServerModel server, RunInstancesRequest req) {
		String subnetUuid = server.getSubnetUuid();
		String networkUuid = server.getNetworkUuid();
		if (null != subnetUuid && null != networkUuid) {
			VirtualPrivateCloud virtualPrivateCloud = new VirtualPrivateCloud();
			virtualPrivateCloud.setSubnetId(server.getSubnetUuid());
			virtualPrivateCloud.setVpcId(server.getNetworkUuid());
			virtualPrivateCloud.setAsVpcGateway(true);
			req.setVirtualPrivateCloud(virtualPrivateCloud);
		}
	}

	private void systemDiskParams(ServerModel server, RunInstancesRequest req) {
		// system disk
		// LOCAL_BASIC：本地硬盘
		// LOCAL_SSD：本地SSD硬盘
		// CLOUD_BASIC：普通云硬盘
		// CLOUD_SSD：SSD云硬盘
		// CLOUD_PREMIUM：高性能云硬盘
		SystemDisk systemDisk = new SystemDisk();
		systemDisk.setDiskSize(server.getSystemDiskSize());
		// 默认普通云硬盘
		systemDisk.setDiskType("CLOUD_PREMIUM");
		req.setSystemDisk(systemDisk);
	}

	private void publicIp(ServerModel server, RunInstancesRequest req) {
		// BANDWIDTH_POSTPAID_BY_HOUR：带宽按小时后付费
		// •TRAFFIC_POSTPAID_BY_HOUR：流量按小时后付费
		String internetChargeType = server.getInternetChargeType();
		if (null == internetChargeType || "".equals(internetChargeType)) {
			internetChargeType = "TRAFFIC_POSTPAID_BY_HOUR";
		}
		InternetAccessible internetAccessible = new InternetAccessible();
		internetAccessible.setInternetChargeType(internetChargeType);
		Boolean isUsePublicIp = server.getIsUsePublicIp();
		if (null == isUsePublicIp) {
			isUsePublicIp = Boolean.TRUE;
		}
		internetAccessible.setPublicIpAssigned(isUsePublicIp);
		Integer internetMaxBandwidthOut = server.getInternetMaxBandwidthOut();
		if (null == internetMaxBandwidthOut || internetMaxBandwidthOut <= 0) {
			internetMaxBandwidthOut = 10;
		}
		internetAccessible.setInternetMaxBandwidthOut(internetMaxBandwidthOut);
		req.setInternetAccessible(internetAccessible);
	}

	private void loginParams(ServerModel server, RunInstancesRequest req) {
		LoginSettings loginSettings = new LoginSettings();
		String password = server.getPassword();
		if (null != password && !"".equals(password)) {
			loginSettings.setPassword(password);
		}
		String keypairId = server.getKeypairName();
		if (null != keypairId && !"".equals(keypairId)) {
			loginSettings.setKeyIds(new String[] { keypairId });
		}
		req.setLoginSettings(loginSettings);
	}

	private void volumeParams(ServerModel server, RunInstancesRequest req) {
		List<VolumeModel> diskDevices = server.getDiskDevices();
		if (null != diskDevices && !diskDevices.isEmpty()) {
			DataDisk disk = null;
			List<DataDisk> diskList = new ArrayList<>();
			for (VolumeModel volumeModel : diskDevices) {
				disk = new DataDisk();
				disk.setDiskSize(volumeModel.getSize());
				diskList.add(disk);
			}
			DataDisk[] dataDisks = new DataDisk[diskList.size()];
			req.setDataDisks(diskList.toArray(dataDisks));
		}
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			if(!StringUtils.isEmpty(server.getRemark())) {
				return new BocloudResult<>(false, "不支持编辑描述信息");
			}
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ModifyInstancesAttributeRequest req = new ModifyInstancesAttributeRequest();
			req.setInstanceIds(new String[] { server.getInstanceId() });
			String name = server.getName();
			if (!StringUtils.isEmpty(name)) {
				req.setInstanceName(name);
			}
			List<String> securityGroups = server.getSecurityGroups();
			if (null != securityGroups && !securityGroups.isEmpty()) {
				String[] groups = new String[securityGroups.size()];
				req.setSecurityGroups(securityGroups.toArray(groups));
			}
			client.ModifyInstancesAttribute(req);
			BocloudResult detail = this.detail(server.getInstanceId());
			return new BocloudResult(true, detail.getData(), "修改成功");
		} catch (Exception e) {
			logger.error("修改失败： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			TerminateInstancesRequest req = new TerminateInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			TerminateInstancesResponse resp = client.TerminateInstances(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			DescribeInstancesRequest req = new DescribeInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			DescribeInstancesResponse resp = client.DescribeInstances(req);
			Instance[] instanceSet = resp.getInstanceSet();
			ServerModel convertModel = null;
			if (null != instanceSet && instanceSet.length > 0) {
				Instance instance = instanceSet[0];
				convertModel = convertor.convertModel(instance);
				convertModel.setRegionId(this.getRegionId());
				List<VolumeModel> volumeDetail = this.getVolumeDetail(convertModel);
				if (null != volumeDetail) {
					convertModel.setDiskDevices(volumeDetail);
				}
				this.getNetworkCardData(convertModel);
			}
			return new BocloudResult(true, convertModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	public void getNetworkCardData(ServerModel convertModel) {
		try {
			TencentNetworkCardProvider tencentNetworkCardProvider = new TencentNetworkCardProvider(butler);
			BocloudResult networkCardResult = tencentNetworkCardProvider.list();
			if (networkCardResult.isFailed()) {
				return;
			}
			List<NetworkCardModel> networkCardModels = (List<NetworkCardModel>) networkCardResult.getData();
			if (ListTool.isEmpty(networkCardModels)) {
				return;
			}
			networkCardModels = networkCardModels.stream()
					.filter(model -> convertModel.getInstanceId().equalsIgnoreCase(model.getInstanceId()))
					.collect(Collectors.toList());
			convertModel.setNetworkCards(networkCardModels);
		} catch (Exception e) {
			logger.error("查询虚机网卡异常：" + e.getMessage());
		}
	}

	private List<VolumeModel> getVolumeDetail(ServerModel convertModel) {
		List<VolumeModel> result = null;
		try {
			TencentVolumeProvider volume = new TencentVolumeProvider(butler);
			Integer count = 3;
			while (count > 0) {
				BocloudResult list = volume.list(convertModel.getInstanceId());
				List<VolumeModel> parseArray = JSONObject.parseArray(JSON.toJSONString(list.getData()),
						VolumeModel.class);
				if (!parseArray.isEmpty()) {
					result = parseArray;
					count = -1;
					break;
				} else {
					count--;
					Thread.sleep(2000);
				}
			}
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
		}
		return result;
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			StartInstancesRequest req = new StartInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.StartInstances(req);
			BocloudResult instances = this.getInstances(instanceId);
			return new BocloudResult(true, instances.getData(), "开机成功");
		} catch (Exception e) {
			logger.error("开机失败 ： " + e);
			return new BocloudResult(false, "开机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			StopInstancesRequest req = new StopInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.StopInstances(req);
			BocloudResult instances = this.getInstances(instanceId);
			return new BocloudResult(true, instances.getData(), "关机成功");
		} catch (Exception e) {
			logger.error("关机失败 ： " + e);
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			RebootInstancesRequest req = new RebootInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.RebootInstances(req);
			BocloudResult instances = this.getInstances(instanceId);
			return new BocloudResult(true, instances.getData(), "重启成功");
		} catch (Exception e) {
			logger.error("开机失败 ： " + e);
			return new BocloudResult(false, "重启失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult pause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult resume(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String group) {
		try {
			// 查询当前实例的安全组
			BocloudResult bocloudResult = this.detail(instanceId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, bocloudResult.getMessage(), "修改安全组失败");
			}
			ServerModel modes = (ServerModel) bocloudResult.getData();
			List<String> securityGroups = new ArrayList<>(modes.getSecurityGroups());
			// 加入安全组
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ModifyInstancesAttributeRequest req = new ModifyInstancesAttributeRequest();
			req.setInstanceIds(new String[] { instanceId });
			securityGroups.add(group);
			String[] strings = new String[securityGroups.size()];
			req.setSecurityGroups(securityGroups.toArray(strings));
			ModifyInstancesAttributeResponse resp = client.ModifyInstancesAttribute(req);
			return new BocloudResult(true, resp.getRequestId(), "安全组加入成功");
		} catch (Exception e) {
			logger.error("安全组加入失败 ： " + e);
			return new BocloudResult(false, "安全组加入失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String group) {
		try {
			// 查询当前实例的安全组
			BocloudResult bocloudResult = this.detail(instanceId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, bocloudResult.getMessage(), "当前实例不存在");
			}
			ServerModel modes = (ServerModel) bocloudResult.getData();
			List<String> asList = new ArrayList<>(modes.getSecurityGroups());
			// 移除安全组
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ModifyInstancesAttributeRequest req = new ModifyInstancesAttributeRequest();
			req.setInstanceIds(new String[] { instanceId });
			asList.remove(group);
			String[] strings = new String[asList.size()];
			req.setSecurityGroups(asList.toArray(strings));
			ModifyInstancesAttributeResponse resp = client.ModifyInstancesAttribute(req);
			return new BocloudResult(true, resp.getRequestId(), "移除安全组成功");
		} catch (Exception e) {
			logger.error("移除安全组失败 ： " + e);
			return new BocloudResult(false, "移除安全组失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		return null;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		return null;
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstancesPasswordRequest req = new ResetInstancesPasswordRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setPassword(password);
			ResetInstancesPasswordResponse resp = client.ResetInstancesPassword(req);
			return new BocloudResult(true, resp.getRequestId(), "重置密码成功");
		} catch (Exception e) {
			logger.error("重置失败 ： " + e);
			return new BocloudResult(false, "重置密码失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			AssociateAddressRequest req = new AssociateAddressRequest();
			req.setInstanceId(server.getInstanceId());
			req.setAddressId(server.getPublicIps());
			AssociateAddressResponse resp = client.AssociateAddress(req);
			return new BocloudResult(true, resp.getRequestId(), "绑定弹性IP成功");
		} catch (Exception e) {
			logger.error("绑定IP失败： " + e);
			return new BocloudResult(false, "绑定弹性IP失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DisassociateAddressRequest req = new DisassociateAddressRequest();
			req.setAddressId(portUuid);
			DisassociateAddressResponse resp = client.DisassociateAddress(req);
			return new BocloudResult(true, resp.getRequestId(), "解绑弹性IP成功");
		} catch (Exception e) {
			logger.error("解绑弹性IP失败： " + e);
			return new BocloudResult(false, "解绑弹性IP失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstancesTypeRequest req = new ResetInstancesTypeRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setInstanceType(flavorId);
			client.ResetInstancesType(req);
			return this.getInstances(instanceId);
		} catch (Exception e) {
			logger.error("修改配置失败： " + e);
			return new BocloudResult(false, "修改配置失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstancesTypeRequest req = new ResetInstancesTypeRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setInstanceType(flavorId);
			req.setForceStop(forceStop);
			client.ResetInstancesType(req);
			return new BocloudResult(true, this.getInstances(instanceId).getData(), "修改配置成功");
		} catch (Exception e) {
			logger.error("修改配置失败： " + e);
			return new BocloudResult(false, "修改配置失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstancesPasswordRequest req = new ResetInstancesPasswordRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setPassword(newPasswd);
			client.ResetInstancesPassword(req);
			BocloudResult detail = this.detail(instanceId);
			return new BocloudResult(true, detail.getData(), "VM重置密码成功");
		} catch (Exception e) {
			logger.error("重置密码失败 ： " + e);
			return new BocloudResult(false, "重置密码失败", e.getMessage());
		}
	}

	/**
	 * 虚拟机磁盘扩容
	 */
	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResizeInstanceDisksRequest request = new ResizeInstanceDisksRequest();
			request.setInstanceId(instanceId);
			request.setForceStop(false); // 默认不强制关机
			DataDisk dataDisk = new DataDisk();
			dataDisk.setDiskSize(extendSize);
			request.setDataDisks(new DataDisk[] { dataDisk });
			ResizeInstanceDisksResponse resizeInstanceDisks = client.ResizeInstanceDisks(request);
			return new BocloudResult(true, resizeInstanceDisks.getRequestId(), "扩容成功");
		} catch (Exception e) {
			logger.error("扩容失败 ： " + e);
			return new BocloudResult(false, "磁盘扩容失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult getMetric(String type) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult list(String hostId) {
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel arg0) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long arg0, String arg1) {
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstanceRequest req = new ResetInstanceRequest();
			req.setInstanceId(serverModel.getInstanceId());
			req.setImageId(serverModel.getImageUuid());
			// 登录方式
			LoginSettings loginSettings = new LoginSettings();
			String password = serverModel.getPassword();
			if (null != password && !"".equals(password)) {
				loginSettings.setPassword(password);
			}
			String keypairId = serverModel.getKeypairName();
			if (null != keypairId && !"".equals(keypairId)) {
				loginSettings.setKeyIds(new String[] { keypairId });
			}
			req.setLoginSettings(loginSettings);
			client.ResetInstance(req);
			return new BocloudResult(true,
					this.getInstancesByImage(serverModel.getInstanceId(), serverModel.getImageUuid()).getData(),
					"系统重装成功");
		} catch (Exception e) {
			logger.error("系统重装失败 ： " + e);
			return new BocloudResult(false, "系统重装失败", e.getMessage());
		}
	}

	public BocloudResult getInstancesByImage(String instanceId, String imageId) {
		BocloudResult result = null;
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				Thread.sleep(3000);
				result = this.detail(instanceId);
				ServerModel serverModel = JSON.parseObject(JSON.toJSONString(result.getData()), ServerModel.class);
				if (null != serverModel && imageId.equals(serverModel.getImageUuid())) {
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					flag = false;
					result = this.detail(instanceId);
					break;
				}
			}
			return result;
		} catch (Exception e) {
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ResetInstancesInternetMaxBandwidthRequest req = new ResetInstancesInternetMaxBandwidthRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setStartTime(format.format(new Date()));
			req.setEndTime(format.format(DateUtils.addDays(new Date(), 365)));
			InternetAccessible internetAccessible = new InternetAccessible();
			internetAccessible.setInternetMaxBandwidthOut(internetMaxBandwidthOut);
			req.setInternetAccessible(internetAccessible);
			ResetInstancesInternetMaxBandwidthResponse resp = client.ResetInstancesInternetMaxBandwidth(req);
			return new BocloudResult(true, resp.getRequestId(), "修改宽带成功");
		} catch (Exception e) {
			logger.error("修改宽带失败 ： " + e);
			return new BocloudResult(false, "修改宽带失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			AssociateInstancesKeyPairsRequest req = new AssociateInstancesKeyPairsRequest();
			req.setInstanceIds(serverModel.getInstanceIds());
			req.setKeyIds(serverModel.getKeypairIds());
			client.AssociateInstancesKeyPairs(req);
			BocloudResult instances = getInstances(serverModel.getInstanceId());
			return new BocloudResult(true, instances.getData(), "绑定成功");
		} catch (Exception e) {
			logger.error("绑定失败 ： " + e);
			return new BocloudResult(false, "绑定失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			DisassociateInstancesKeyPairsRequest req = new DisassociateInstancesKeyPairsRequest();
			req.setInstanceIds(serverModel.getInstanceIds());
			req.setKeyIds(serverModel.getKeypairIds());
			DisassociateInstancesKeyPairsResponse resp = client.DisassociateInstancesKeyPairs(req);
			BocloudResult bocloudResult = queryServerStatus(serverModel.getInstanceIds());
			return new BocloudResult(bocloudResult.isSuccess(), resp.getRequestId(), "解绑成功");
		} catch (Exception e) {
			logger.error("解绑失败 ： " + e);
			return new BocloudResult(false, "解绑失败", e.getMessage());
		}
	}

	public Instance[] listInstances(String[] instanceIds) throws Exception {
		if (null == instanceIds || instanceIds.length <= 0) {
			throw new RuntimeException("实例不存在！");
		}
		DescribeInstancesRequest req = new DescribeInstancesRequest();
		req.setInstanceIds(instanceIds);
		DescribeInstancesResponse response = ((CvmClient) this.getClient(TencentEnum.CVM)).DescribeInstances(req);
		return response.getInstanceSet();
	}

	public BocloudResult queryServerStatus(String[] instanceIds) {
		BocloudResult result = null;
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				Thread.sleep(3000);
				Instance[] instanceSet = this.listInstances(instanceIds);
				if (null != instanceSet && instanceSet.length == instanceIds.length) {
					for (Instance instance : instanceSet) {
						flag = "STOPPED".equalsIgnoreCase(instance.getInstanceState());
						if (!flag) {
							flag = !flag;
							break;
						}
					}
					result = new BocloudResult(true, "查询成功！");
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					flag = false;
					result = new BocloudResult(false, "实例不存在！");
					break;
				}
			}
			return result;
		} catch (Exception e) {
			return new BocloudResult(false, "查询状态失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		return null;
	}

}