package com.bocloud.cmp.provider.compute;

import java.util.Arrays;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.SnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.cbs.v20170312.models.ApplySnapshotRequest;
import com.tencentcloudapi.cbs.v20170312.models.ApplySnapshotResponse;
import com.tencentcloudapi.cbs.v20170312.models.DeleteSnapshotsRequest;
import com.tencentcloudapi.cbs.v20170312.models.DeleteSnapshotsResponse;
import com.tencentcloudapi.cbs.v20170312.models.DescribeSnapshotsRequest;
import com.tencentcloudapi.cbs.v20170312.models.DescribeSnapshotsResponse;
import com.tencentcloudapi.cbs.v20170312.models.ModifySnapshotAttributeRequest;
import com.tencentcloudapi.cbs.v20170312.models.ModifySnapshotAttributeResponse;
import com.tencentcloudapi.cbs.v20170312.models.Snapshot;

/**
 * 快照操作类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentSnapshotProvider extends TencentProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentSnapshotProvider.class);

    private BeanConvertor<SnapshotModel, Snapshot> convertor = new SnapshotConvertor();

    public TencentSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSnapshotsRequest req = new DescribeSnapshotsRequest();
            CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
            DescribeSnapshotsResponse resp = client.DescribeSnapshots(req);
            Snapshot[] snapshotSet = resp.getSnapshotSet();
            List<SnapshotModel> convertList = convertor.convertList(Arrays.asList(snapshotSet));
            for (SnapshotModel snapshotModel : convertList) {
                snapshotModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询列表成功");
        } catch (Exception e) {
            return new BocloudResult(false, "查询镜像失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
            DeleteSnapshotsRequest req = new DeleteSnapshotsRequest();
            req.setSnapshotIds(new String[] { model.getSnapshotUuid() });
            DeleteSnapshotsResponse resp = client.DeleteSnapshots(req);
            return new BocloudResult(true, resp.getRequestId(), "删除成功");
        } catch (Exception e) {
            return new BocloudResult(false, "删除失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
            DescribeSnapshotsRequest req = new DescribeSnapshotsRequest();
            req.setSnapshotIds(new String[] { model.getSnapshotUuid() });
            DescribeSnapshotsResponse resp = client.DescribeSnapshots(req);
            Snapshot[] snapshotSet = resp.getSnapshotSet();
            SnapshotModel convertModel = null;
            if (null != snapshotSet && snapshotSet.length > 0) {
                convertModel = convertor.convertModel(snapshotSet[0]);
                convertModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertModel, "查询成功");
        } catch (Exception e) {
            return new BocloudResult(false, "查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
            ModifySnapshotAttributeRequest req = new ModifySnapshotAttributeRequest();
            req.setSnapshotId(model.getSnapshotUuid());
            req.setSnapshotName(model.getName());
            ModifySnapshotAttributeResponse resp = client.ModifySnapshotAttribute(req);
            return new BocloudResult(true, resp.getRequestId(), "修改成功");
        } catch (Exception e) {
            return new BocloudResult(false, "修改失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
            ApplySnapshotRequest req = new ApplySnapshotRequest();
            req.setDiskId(model.getVolumeUuid());
            req.setSnapshotId(model.getSnapshotUuid());
            ApplySnapshotResponse resp = client.ApplySnapshot(req);
            return new BocloudResult(true, resp.getRequestId(), "回滚成功");
        } catch (Exception e) {
            return new BocloudResult(false, "回滚失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        Boolean flag = true;
        BocloudResult result = new BocloudResult(false, "查询失败");
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            SnapshotModel model = new SnapshotModel();
            model.setSnapshotUuid(snapshotId);
            BocloudResult detail = this.detail(model);
            if (detail.isSuccess()) {
                SnapshotModel snapshot = (SnapshotModel) detail.getData();
                if (null == snapshot) {
                    logger.error("snapshot [{}] doesnot exsit!", snapshotId);
                    return new BocloudResult(false, "快照不存在！");
                }
                String snapshotState = snapshot.getStatus();
                if ("NORMAL".equalsIgnoreCase(snapshotState) || "AVAILABLE".equalsIgnoreCase(snapshotState)) {
                    flag = false;
                    result = detail;
                    break;
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 5 * 60 * 1000) {
                flag = false;
                result = detail;
                break;
            }
        }
        return result;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long arg0) {
        return null;
    }

}
