package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.custom.CustomVpcClient;
import com.bocloud.cmp.custom.models.CustomAddress;
import com.bocloud.cmp.custom.models.CustomDescribeAddressesResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.FloatingIpConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * tencent浮动IP操作类
 *
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentFloatingIpProvider extends TencentProvider implements FloatingIPProvider {

	public TencentFloatingIpProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(TencentFloatingIpProvider.class);

	private BeanConvertor<FloatingIpModel, CustomAddress> convertor = new FloatingIpConvertor();

	@Override
	public BocloudResult list() {
		try {
			CustomVpcClient client = (CustomVpcClient) this.getClient(TencentEnum.VPC);
			DescribeAddressesRequest req = new DescribeAddressesRequest();
			CustomDescribeAddressesResponse resp = client.customDescribeAddresses(req);
			CustomAddress[] addressSet = resp.getAddressSet();
			List<FloatingIpModel> convertList = convertor.convertList(Arrays.asList(addressSet));
			for (FloatingIpModel floatingIpModel : convertList) {
				floatingIpModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	private BocloudResult list(String[] addressIds) {
		try {
			CustomVpcClient client = (CustomVpcClient) this.getClient(TencentEnum.VPC);
			DescribeAddressesRequest req = new DescribeAddressesRequest();
			if (null != addressIds && addressIds.length > 0) {
				req.setAddressIds(addressIds);
			}
			CustomDescribeAddressesResponse resp = client.customDescribeAddresses(req);
			CustomAddress[] addressSet = resp.getAddressSet();
			List<FloatingIpModel> convertList = convertor.convertList(Arrays.asList(addressSet));
			for (FloatingIpModel floatingIpModel : convertList) {
				floatingIpModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	public FloatingIpModel getFloatingIpDetail(String addressId) {
		FloatingIpModel result = null;
		try {
			CustomVpcClient client = (CustomVpcClient) this.getClient(TencentEnum.VPC);
			DescribeAddressesRequest req = new DescribeAddressesRequest();
			req.setAddressIds(new String[] { addressId });
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				CustomDescribeAddressesResponse resp = client.customDescribeAddresses(req);
				CustomAddress[] addressSet = resp.getAddressSet();
				if (addressSet == null || addressSet.length <= 0) {
					flag = false;
					break;
				}
				CustomAddress address = addressSet[0];
				if ("UNBIND".equalsIgnoreCase(address.getAddressStatus())) {
					flag = false;
					result = convertor.convertModel(address);
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					flag = false;
					result = convertor.convertModel(address);
					break;
				}
				Thread.sleep(2000);
			}
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
		}
		return result;
	}

	@Override
	public BocloudResult create(FloatingIpModel model) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			AllocateAddressesRequest req = new AllocateAddressesRequest();
			req.setAddressCount(1);
			AllocateAddressesResponse resp = client.AllocateAddresses(req);
			String[] addressSet = resp.getAddressSet();
			List<FloatingIpModel> models = new ArrayList<>();
			for (String ipId : addressSet) {
				FloatingIpModel floatingIpDetail = this.getFloatingIpDetail(ipId);
				if (null != floatingIpDetail) {
					floatingIpDetail.setRegionId(this.getRegionId());
					models.add(floatingIpDetail);
				}
			}
			return new BocloudResult(true, models, "弹性IP创建成功");
		} catch (Exception e) {
			return new BocloudResult(false, "弹性IP创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String fipId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			ReleaseAddressesRequest req = new ReleaseAddressesRequest();
			req.setAddressIds(new String[] { fipId });
			ReleaseAddressesResponse resp = client.ReleaseAddresses(req);
			return new BocloudResult(true, resp.getRequestId(), "释放IP成功");
		} catch (Exception e) {
			return new BocloudResult(false, "释放IP失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String fipId) {
		return null;
	}

	@Override
	public BocloudResult associate(FloatingIpModel modeld) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			AssociateAddressRequest req = new AssociateAddressRequest();
			String instanceUuid = modeld.getInstanceId();
			if (null != instanceUuid && !"".equals(instanceUuid)) {
				req.setInstanceId(instanceUuid);
			}
			String networkCardUuid = modeld.getNetworkCardUuid();
			if (null != networkCardUuid && !"".equals(networkCardUuid)) {
				req.setNetworkInterfaceId(networkCardUuid);
				req.setPrivateIpAddress(modeld.getPrivateIpAddress());
			}
			req.setAddressId(modeld.getFloatingIpUuid());
			client.AssociateAddress(req);
			return new BocloudResult(true, this.getAssociateFloatingIpDetail(modeld.getFloatingIpUuid()), "绑定成功");
		} catch (Exception e) {
			return new BocloudResult(false, "绑定失败", e.getMessage());
		}
	}

	public FloatingIpModel getAssociateFloatingIpDetail(String addressId) {
		FloatingIpModel result = null;
		try {
			CustomVpcClient client = (CustomVpcClient) this.getClient(TencentEnum.VPC);
			DescribeAddressesRequest req = new DescribeAddressesRequest();
			req.setAddressIds(new String[] { addressId });
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				CustomDescribeAddressesResponse resp = client.customDescribeAddresses(req);
				CustomAddress[] addressSet = resp.getAddressSet();
				if (addressSet == null || addressSet.length <= 0) {
					flag = false;
					break;
				}
				CustomAddress address = addressSet[0];
				if ("BIND".equalsIgnoreCase(address.getAddressStatus())
						|| "BIND_ENI".equalsIgnoreCase(address.getAddressStatus()) && null != address.getInstanceId()) {
					flag = false;
					result = convertor.convertModel(address);
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					flag = false;
					result = convertor.convertModel(address);
					break;
				}
				Thread.sleep(2000);
			}
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
		}
		return result;
	}

	@Override
	public BocloudResult associate(String fipId, String portId) {
		return null;
	}

	@Override
	public BocloudResult disassociate(FloatingIpModel modeld) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DisassociateAddressRequest req = new DisassociateAddressRequest();
			req.setAddressId(modeld.getFloatingIpUuid());
			client.DisassociateAddress(req);
			return new BocloudResult(true, this.getFloatingIpDetail(modeld.getFloatingIpUuid()), "解绑成功成功");
		} catch (Exception e) {
			return new BocloudResult(false, "解绑失败失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(String fipId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DisassociateAddressRequest req = new DisassociateAddressRequest();
			req.setAddressId(fipId);
			client.DisassociateAddress(req);
			return new BocloudResult(true, this.getFloatingIpDetail(fipId), "解绑成功成功");
		} catch (Exception e) {
			return new BocloudResult(false, "解绑失败失败", e.getMessage());
		}
	}

}
