package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.NetworkCardConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.AttachNetworkInterfaceRequest;
import com.tencentcloudapi.vpc.v20170312.models.CreateNetworkInterfaceRequest;
import com.tencentcloudapi.vpc.v20170312.models.CreateNetworkInterfaceResponse;
import com.tencentcloudapi.vpc.v20170312.models.DeleteNetworkInterfaceRequest;
import com.tencentcloudapi.vpc.v20170312.models.DeleteNetworkInterfaceResponse;
import com.tencentcloudapi.vpc.v20170312.models.DescribeNetworkInterfacesRequest;
import com.tencentcloudapi.vpc.v20170312.models.DescribeNetworkInterfacesResponse;
import com.tencentcloudapi.vpc.v20170312.models.DetachNetworkInterfaceRequest;
import com.tencentcloudapi.vpc.v20170312.models.NetworkInterface;
import com.tencentcloudapi.vpc.v20170312.models.PrivateIpAddressSpecification;

/**
 * @author 作者
 * @version 创建时间：2018年12月21日 下午3:06:11 类说明
 */
@Provider("bocloud.cmp.tencent")
public class TencentNetworkCardProvider extends TencentProvider implements NetworkCardProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentNetworkCardProvider.class);

    private BeanConvertor<NetworkCardModel, NetworkInterface> convertor = new NetworkCardConvertor();

    public TencentNetworkCardProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DescribeNetworkInterfacesRequest req = new DescribeNetworkInterfacesRequest();
            DescribeNetworkInterfacesResponse resp = client.DescribeNetworkInterfaces(req);
            NetworkInterface[] networkInterfaceSet = resp.getNetworkInterfaceSet();
            List<NetworkCardModel> convertList = new ArrayList<>();
            if (null != networkInterfaceSet && networkInterfaceSet.length > 0) {
                List<NetworkInterface> networkCarkList = Arrays.asList(networkInterfaceSet);
                convertList = convertor.convertList(networkCarkList);
                for (NetworkCardModel networkCardModel : convertList) {
                    networkCardModel.setRegionId(this.getRegionId());
                }
            }
            return new BocloudResult(true, convertList, "网卡查询成功");
        } catch (Exception e) {
            return new BocloudResult(false, "网卡查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String networkCardId) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DescribeNetworkInterfacesRequest req = new DescribeNetworkInterfacesRequest();
            req.setNetworkInterfaceIds(new String[] { networkCardId });
            DescribeNetworkInterfacesResponse resp = client.DescribeNetworkInterfaces(req);
            NetworkInterface[] networkInterfaceSet = resp.getNetworkInterfaceSet();
            NetworkInterface networkInterface = networkInterfaceSet[0];
            NetworkCardModel convertModel = convertor.convertModel(networkInterface);
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "网卡查询成功");
        } catch (Exception e) {
            logger.error("查询网卡失败：" + e);
            return new BocloudResult(false, "网卡查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult attachNetworkCard(NetworkCardModel networkCardModel) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            AttachNetworkInterfaceRequest req = new AttachNetworkInterfaceRequest();
            req.setInstanceId(networkCardModel.getInstanceId());
            req.setNetworkInterfaceId(networkCardModel.getNetworkcardUuid());
            client.AttachNetworkInterface(req);
            return new BocloudResult(true, this.getNetworkCardDetail(networkCardModel.getNetworkcardUuid()).getData(),
                    "网卡绑定成功");
        } catch (Exception e) {
            return new BocloudResult(false, "网卡绑定失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detachNetworkCard(NetworkCardModel networkCardModel) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DetachNetworkInterfaceRequest req = new DetachNetworkInterfaceRequest();
            req.setInstanceId(networkCardModel.getInstanceId());
            req.setNetworkInterfaceId(networkCardModel.getNetworkcardUuid());
            client.DetachNetworkInterface(req);
            return new BocloudResult(true, this.getNetworkCardDetail(networkCardModel.getNetworkcardUuid()).getData(),
                    "网卡解绑成功");
        } catch (Exception e) {
            return new BocloudResult(false, "网卡解绑失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String networkCardId) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DeleteNetworkInterfaceRequest req = new DeleteNetworkInterfaceRequest();
            req.setNetworkInterfaceId(networkCardId);
            DeleteNetworkInterfaceResponse resp = client.DeleteNetworkInterface(req);
            return new BocloudResult(true, resp.getRequestId(), "删除网卡成功");
        } catch (Exception e) {
            return new BocloudResult(false, "删除网卡失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkCardModel networkCardModel) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            CreateNetworkInterfaceRequest req = new CreateNetworkInterfaceRequest();
            req.setNetworkInterfaceName(networkCardModel.getName());
            req.setNetworkInterfaceDescription(networkCardModel.getRemark());
            req.setVpcId(networkCardModel.getVpcUuid());
            req.setSubnetId(networkCardModel.getSubnetUuid());
            // 指定内网信息
            List<JSONObject> privateIpAddressSet = networkCardModel.getPrivateIpAddressSet();
            if (null != privateIpAddressSet && !privateIpAddressSet.isEmpty()) {
                PrivateIpAddressSpecification[] privateIpAddressesArray = new PrivateIpAddressSpecification[privateIpAddressSet
                        .size()];
                PrivateIpAddressSpecification privateIpAddressSpecification = null;
                for (int i = 0; i < privateIpAddressSet.size(); i++) {
                    privateIpAddressSpecification = new PrivateIpAddressSpecification();
                    JSONObject privateIpAddrObj = privateIpAddressSet.get(i);
                    privateIpAddressSpecification.setDescription(privateIpAddrObj.getString("description"));
                    privateIpAddressSpecification.setPrivateIpAddress(privateIpAddrObj.getString("privateIpAddress"));
                    privateIpAddressSpecification.setPrimary(privateIpAddrObj.getBoolean("primary"));
                    privateIpAddressSpecification.setPublicIpAddress(privateIpAddrObj.getString("publicIpAddress"));
                    privateIpAddressesArray[i] = privateIpAddressSpecification;
                }
                if (privateIpAddressesArray.length > 0) {
                    req.setPrivateIpAddresses(privateIpAddressesArray);
                }
            }
            CreateNetworkInterfaceResponse resp = client.CreateNetworkInterface(req);
            NetworkInterface networkInterface = resp.getNetworkInterface();
            return new BocloudResult(true, getNetworkCardDetail(networkInterface.getNetworkInterfaceId()).getData(),
                    "创建网卡成功");
        } catch (Exception e) {
            return new BocloudResult(false, "创建网卡失败", e.getMessage());
        }
    }

    public BocloudResult getNetworkCardDetail(String networkCardId) {
        BocloudResult result = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(5000);
                VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
                DescribeNetworkInterfacesRequest req = new DescribeNetworkInterfacesRequest();
                req.setNetworkInterfaceIds(new String[] { networkCardId });
                DescribeNetworkInterfacesResponse resp = client.DescribeNetworkInterfaces(req);
                NetworkInterface[] networkInterfaceSet = resp.getNetworkInterfaceSet();
                if (networkInterfaceSet == null || networkInterfaceSet.length <= 0) {
                    flag = false;
                    result = new BocloudResult(false, "失败");
                    break;
                }
                NetworkInterface networkInterface = networkInterfaceSet[0];
                if (null != networkInterface.getCreatedTime()
                        && !networkInterface.getCreatedTime().equals("0000-00-00 00:00:00")
                        && "AVAILABLE".equalsIgnoreCase(networkInterface.getState())) {
                    flag = false;
                    NetworkCardModel convertModel = convertor.convertModel(networkInterface);
                    convertModel.setRegionId(this.getRegionId());
                    result = new BocloudResult(true, convertModel, "success");
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 3 * 60 * 1000) {
                    flag = false;
                    result = this.detail(networkCardId);
                    break;
                }
            }
            return result;
        } catch (Exception e) {
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }

}
