package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.RouteTableConvertor;
import com.bocloud.cmp.tencentconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.DescribeRouteTablesRequest;
import com.tencentcloudapi.vpc.v20170312.models.DescribeRouteTablesResponse;
import com.tencentcloudapi.vpc.v20170312.models.RouteTable;
import com.tencentcloudapi.vpc.v20170312.models.Subnet;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;

/**
 * 路由表操作类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
@Slf4j
public class TencentRouteTableProvider extends TencentProvider implements RouteTableProvider {
    private BeanConvertor<RouteTableModel, RouteTable> convertor = new RouteTableConvertor();

    public TencentRouteTableProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DescribeRouteTablesRequest req = new DescribeRouteTablesRequest();
            DescribeRouteTablesResponse resp = client.DescribeRouteTables(req);
            RouteTable[] routeTableSet = resp.getRouteTableSet();
            return new BocloudResult(true, convertor.convertList(Arrays.asList(routeTableSet)), "查询成功");
        } catch (Exception e) {
            log.error("查询列表失败 ： " + e);
            return new BocloudResult(false, "查询列表失败", e.getMessage());
        }
    }

}
