package com.bocloud.cmp.provider.network;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.CreateSubnetRequest;
import com.tencentcloudapi.vpc.v20170312.models.CreateSubnetResponse;
import com.tencentcloudapi.vpc.v20170312.models.DeleteSubnetRequest;
import com.tencentcloudapi.vpc.v20170312.models.DeleteSubnetResponse;
import com.tencentcloudapi.vpc.v20170312.models.DescribeSubnetsRequest;
import com.tencentcloudapi.vpc.v20170312.models.DescribeSubnetsResponse;
import com.tencentcloudapi.vpc.v20170312.models.ModifySubnetAttributeRequest;
import com.tencentcloudapi.vpc.v20170312.models.ModifySubnetAttributeResponse;
import com.tencentcloudapi.vpc.v20170312.models.Subnet;

/**
 * 子网操作类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentSubnetProvider extends TencentProvider implements SubnetProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentSubnetProvider.class);

	private BeanConvertor<SubnetModel, Subnet> convertor = new SubnetConvertor();

	public TencentSubnetProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DescribeSubnetsRequest req = new DescribeSubnetsRequest();
			DescribeSubnetsResponse resp = client.DescribeSubnets(req);
			Subnet[] subnetSet = resp.getSubnetSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(subnetSet)), "查询成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SubnetModel subnet) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			CreateSubnetRequest req = new CreateSubnetRequest();
			req.setVpcId(subnet.getVpcUuid());
			req.setSubnetName(subnet.getName());
			req.setCidrBlock(subnet.getCidr());
			req.setZone(subnet.getZone());
			CreateSubnetResponse resp = client.CreateSubnet(req);
			Object data = this.getSubnetDetail(resp.getSubnet().getSubnetId(), this.getRegionId()).getData();
			SubnetModel convertModel = JSON.parseObject(JSON.toJSONString(data), SubnetModel.class);
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败 ： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	public BocloudResult getSubnetDetail(String subnetId, String regionId) {
		BocloudResult result = null;
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DescribeSubnetsRequest req = new DescribeSubnetsRequest();
			req.setSubnetIds(new String[] { subnetId });
			for (int i = 0; i < 3 * 60; i++) {
				Thread.sleep(1000);
				DescribeSubnetsResponse resp = client.DescribeSubnets(req);
				Subnet[] subnetSet = resp.getSubnetSet();
				if (subnetSet == null || subnetSet.length <= 0) {
					result = new BocloudResult(false, "失败");
					break;
				}
				Subnet subnet = subnetSet[0];
				if (null != subnet.getCreatedTime() && !"".equals("0000-00-00 00:00:00")) {
					result = this.detail(subnetId);
					break;
				}
			}
			return result;
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
			return new BocloudResult(false, "查询状态失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SubnetModel subnet) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			ModifySubnetAttributeRequest req = new ModifySubnetAttributeRequest();
			req.setSubnetId(subnet.getSubnetUuid());
			req.setSubnetName(subnet.getName());
			ModifySubnetAttributeResponse resp = client.ModifySubnetAttribute(req);
			return new BocloudResult(true, resp.getRequestId(), "修改成功");
		} catch (Exception e) {
			logger.error("修改失败 ： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String subnetId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DeleteSubnetRequest req = new DeleteSubnetRequest();
			req.setSubnetId(subnetId);
			DeleteSubnetResponse resp = client.DeleteSubnet(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String subnetId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DescribeSubnetsRequest req = new DescribeSubnetsRequest();
			req.setSubnetIds(new String[] { subnetId });
			DescribeSubnetsResponse resp = client.DescribeSubnets(req);
			Subnet subnet = null;
			Subnet[] subnetSet = resp.getSubnetSet();
			if (null != subnetSet && subnetSet.length > 0) {
				subnet = subnetSet[0];
			}
			return new BocloudResult(true, convertor.convertModel(subnet), "查询成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DeleteSubnetRequest req = new DeleteSubnetRequest();
			req.setSubnetId(subnet.getSubnetUuid());
			DeleteSubnetResponse resp = client.DeleteSubnet(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

}
