package com.bocloud.cmp.provider.storage;


import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.tencent.storage.FileSystemModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tencent.storage.FileSystemProvider;
import com.bocloud.cmp.tencentCfs.v20190719.CfsClient;
import com.bocloud.cmp.tencentCfs.v20190719.models.*;
import com.bocloud.cmp.tencentconvertor.FileSystemConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.List;

@Provider("bocloud.cmp.tencent")
public class TencentFileSystemsProvider extends TencentProvider implements FileSystemProvider {
    private static Logger logger = LoggerFactory.getLogger(TencentFileSystemsProvider.class);

    private BeanConvertor<FileSystemModel, FileSystemInfo> convertor = new FileSystemConvertor();

    public TencentFileSystemsProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(){
        try{
            DescribeCfsFileSystemsRequest req = new DescribeCfsFileSystemsRequest();
            CfsClient client = (CfsClient) this.getClient(TencentEnum.CFS);
            DescribeCfsFileSystemsResponse resp = client.DescribeCfsFileSystems(req);
            FileSystemInfo[] fileSystems = resp.getFileSystems();//取到文件系统基本信息
            List<FileSystemInfo> fileSystemInfos = Arrays.asList(fileSystems);//转乘list
            JSONArray jsonArray = new JSONArray();
            for (int i = 0; i < fileSystemInfos.size(); i++) {
                String fileSystemId = fileSystemInfos.get(i).getFileSystemId();
                DescribeMountTargetsRequest request = new DescribeMountTargetsRequest();
                request.setFileSystemId(fileSystemId);
                DescribeMountTargetsResponse mountResp=client.DescribeMountTargets(request);
                MountInfo[] mountTargets = mountResp.getMountTargets();
                List<MountInfo> mountInfos = Arrays.asList(mountTargets);
                FileSystemModel model = new FileSystemModel();
                for (int j = 0; j <mountInfos.size(); j++) {
                    model.setFileSystemId(fileSystemInfos.get(i).getFileSystemId());//文件存储id
                    String name= fileSystemInfos.get(i).getCreationToken();
                    if (null !=name && "未命名".equals(name)){
                        name = fileSystemInfos.get(i).getFileSystemId();
                    }
                    model.setFileSystemName(name);//文件存储名称
                    model.setCreateTime(fileSystemInfos.get(i).getCreationTime());//创建时间
                    model.setFileSystemStatus(fileSystemInfos.get(i).getLifeCycleState());//文件系统状态
                    model.setUseByte(fileSystemInfos.get(i).getSizeByte());//已使用的容量
                    model.setSpaceConstraintsMax(fileSystemInfos.get(i).getSizeLimit());//文件系统最大空间限制
                    model.setZoneId(fileSystemInfos.get(i).getZoneId());//区域id
                    model.setAvailabilityZone(fileSystemInfos.get(i).getZone());//可用区域名称
                    model.setProtocol(fileSystemInfos.get(i).getProtocol());//协议类型
                    model.setStorageType(fileSystemInfos.get(i).getStorageType());//存储类型
                    model.setPermissionsId(fileSystemInfos.get(i).getPGroup().getPGroupId());//权限id
                    model.setPermissionsName(fileSystemInfos.get(i).getPGroup().getName());//权限名称
                    model.setFsName(fileSystemInfos.get(i).getFsName());
                    model.setEncryption(fileSystemInfos.get(i).getEncrypted());//文件是否加密
                    model.setEncryptionKeyId(fileSystemInfos.get(i).getKmsKeyId());//加密的密钥
                    model.setAppId(fileSystemInfos.get(i).getAppId());//应用Id
                    model.setMaximumThroughput(fileSystemInfos.get(i).getBandwidthLimit());//吞吐上限
                    model.setFileSystemSizeTotal(fileSystemInfos.get(i).getCapacity());//文件系统总容量
                    model.setIp(mountInfos.get(j).getIpAddress());//ip地址
                    model.setVpcId(mountInfos.get(j).getVpcId());//所属网络
                }
                jsonArray.add(model);
            }
            return new BocloudResult(true,jsonArray,"查询成功");
        } catch (TencentCloudSDKException e) {
            e.printStackTrace();
            return new BocloudResult(false,"查询失败",e.getMessage());
        }
    }
}
