package com.bocloud.cmp.tencentconvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.billing.v20180709.models.BillResourceSummary;

/**
 * 虚拟机数据转换器
 */
public class BillingConvertor implements BeanConvertor<PublicCloudBillModel, BillResourceSummary> {

	@Override
	public List<PublicCloudBillModel> convertList(List<? extends BillResourceSummary> objects) {
		List<PublicCloudBillModel> vpcModels = new ArrayList<>();
		for (BillResourceSummary object : objects) {
			vpcModels.add(convertModel(object));
		}
		return vpcModels;
	}

	@Override
	public PublicCloudBillModel convertModel(BillResourceSummary item) {
		PublicCloudBillModel model = null;
		if (null != item) {
			model = new PublicCloudBillModel();

			model.setOrderSn(item.getResourceId());
			model.setMoney(new BigDecimal(item.getRealTotalCost()));
			model.setProductDetail(item.getProductCodeName());
			model.setProductName(item.getBusinessCodeName());
			model.setRegionName(item.getRegionName());
			model.setZoneName(item.getZoneName());
			model.setResourceId(item.getResourceId());
			
			String usageStartTime = item.getFeeBeginTime();
			if (!StringUtils.isEmpty(usageStartTime)) {
				model.setFeeBeginTime(DateTools.stringToDateTime(usageStartTime));
			}
			String usageEndTime = item.getFeeEndTime();
			if (!StringUtils.isEmpty(usageEndTime)) {
				model.setFeeEndTime(DateTools.stringToDateTime(usageEndTime));
			}
		}
		return model;
	}

}
