package com.bocloud.cmp.tencentconvertor;

import com.bocloud.cmp.common.enums.TencentConstants;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.tencentcloudapi.cvm.v20170312.models.Image;

import java.util.ArrayList;
import java.util.List;

/**
 * 镜像转换器
 *
 * @author ymh
 * @时间 2018年11月16日
 */
public class ImageConvertor implements BeanConvertor<ImageModel, Image> {

    @Override
    public List<ImageModel> convertList(List<? extends Image> objects) {
        List<ImageModel> imageModels = new ArrayList<>();
        for (Image object : objects) {
            imageModels.add(convertModel(object));
        }
        return imageModels;
    }

    @Override
    public ImageModel convertModel(Image image) {
        ImageModel model = new ImageModel();
        model.setImageUuid(image.getImageId());
        model.setName(image.getImageName());
        String imageType = image.getImageType();
        if (null != imageType) {
            model.setImageType(convertType(imageType));
        }
        model.setOsName(image.getOsName());
        String platform = image.getPlatform();
        model.setOsType(platform);
        if (null != platform && "Windows".equals(platform)) {
            model.setOsCategory("Windows");
        } else {
            model.setOsCategory("Linux");
        }
        model.setRemark(image.getImageDescription());
        model.setSize(Double.valueOf(image.getImageSize()));
        model.setMinDisk(Long.parseLong(image.getImageSize() + ""));
        model.setArchitecture(image.getArchitecture());
        String imageState = image.getImageState();
        if ("NORMAL".equals(imageState)) {
            model.setStatus("ACTIVE");
        }
        return model;
    }

    public String convertType(String imageType) {
        if (TencentConstants.PUBLIC_IMAGE.equalsIgnoreCase(imageType)) {
            return "PUBLIC";
        }
        if (TencentConstants.PRIVATE_IMAGE.equalsIgnoreCase(imageType)) {
            return "PRIVATE";
        }
        return imageType == null ? "OTHER" : imageType.toUpperCase();
    }
}
