package com.bocloud.cmp.tencentconvertor;

import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.LbListenerRulesModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.tencentcloudapi.clb.v20180317.models.HealthCheck;
import com.tencentcloudapi.clb.v20180317.models.Listener;
import com.tencentcloudapi.clb.v20180317.models.RuleOutput;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 监听器
 *
 * @author ymh
 * @时间 2018年11月22日
 */
public class ListenerConvertor implements BeanConvertor<LbListenerModel, Listener> {

	@Override
	public List<LbListenerModel> convertList(List<? extends Listener> objects) {
		List<LbListenerModel> models = new ArrayList<LbListenerModel>();
		for (Listener object : objects) {
			models.add(convertModel(object));
		}
		return models;
	}

	@Override
	public LbListenerModel convertModel(Listener info) {
		LbListenerModel model = new LbListenerModel();
		if (null != info) {
			model.setListenerUuid(info.getListenerId());
			model.setName(info.getListenerName());
			model.setPort(info.getPort());
			model.setProtocol(info.getProtocol());
			model.setSniSwitch(info.getSniSwitch());
			RuleOutput[] rules = info.getRules();
			if (null != rules && rules.length > 0) {
				List<LbListenerRulesModel> convertRulesList = convertRulesList(Arrays.asList(rules));
				if (null != convertRulesList && convertRulesList.size() > 0) {
					model.setListenerRulesModel(convertRulesList);
				}
			}
		}
		return model;
	}

	private List<LbListenerRulesModel> convertRulesList(List<? extends RuleOutput> objects) {
		List<LbListenerRulesModel> models = new ArrayList<LbListenerRulesModel>();
		for (RuleOutput object : objects) {
			models.add(convertModel(object));
		}
		return models;
	}

	private LbListenerRulesModel convertModel(RuleOutput object) {
		LbListenerRulesModel rulesModel = new LbListenerRulesModel();
		if (null != object) {
			rulesModel.setDomain(object.getDomain());
			rulesModel.setLocationId(object.getLocationId());
			rulesModel.setScheduler(object.getScheduler());
			rulesModel.setSessionExpireTime(object.getSessionExpireTime());
			rulesModel.setUrl(object.getUrl());
			HealthCheck healthCheck = object.getHealthCheck();
			if (null != healthCheck) {
				rulesModel.setHealthNum(healthCheck.getHealthNum());
				rulesModel.setHealthSwitch(healthCheck.getHealthSwitch());
				rulesModel.setHttpCheckDomain(healthCheck.getHttpCheckDomain());
				rulesModel.setHttpCheckMethod(healthCheck.getHttpCheckMethod());
				rulesModel.setHttpCheckPath(healthCheck.getHttpCheckPath());
				rulesModel.setHttpCode(healthCheck.getHttpCode());
				rulesModel.setIntervalTime(healthCheck.getIntervalTime());
				rulesModel.setTimeOut(healthCheck.getTimeOut());
				rulesModel.setUnHealthNum(healthCheck.getUnHealthNum());
			}
		}
		return rulesModel;
	}
}
