package com.bocloud.cmp.tencentconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.clb.v20180317.models.LoadBalancer;

/**
 * 负载均衡 转换
 *
 * @author ymh
 * @时间 2018年11月22日
 */
public class LoadBalanceConvertor implements BeanConvertor<LbModel, LoadBalancer> {

    @Override
    public List<LbModel> convertList(List<? extends LoadBalancer> objects) {
        List<LbModel> loadBalancerModels = new ArrayList<LbModel>();
        for (LoadBalancer object : objects) {
            loadBalancerModels.add(convertModel(object));
        }
        return loadBalancerModels;
    }

    @Override
    public LbModel convertModel(LoadBalancer info) {
        LbModel model = new LbModel();
        if (null != info) {
            model.setForward(info.getForward());
            model.setVpcUuid(info.getVpcId());
            model.setName(info.getLoadBalancerName());
            model.setLbUuid(info.getLoadBalancerId());
            model.setStatus(1 == info.getStatus() ? "ACTIVE" : "UNACTIVE");
            model.setSubnetUuid(info.getSubnetId());
            model.setLoadBalancerType(info.getLoadBalancerType());
            String createTime = info.getCreateTime();
            if (!StringUtils.isEmpty(createTime)) {
                model.setCreateTime(DateTools.stringToDateTime(createTime));
            }
            model.setLoadBalancerVips(JSON.toJSONString(info.getLoadBalancerVips()));
            model.setLog(info.getLog());
            model.setDomain(info.getDomain());
            model.setSnat(info.getSnat());
            model.setIsolation(info.getIsolation());
        }
        return model;
    }
}
