package com.bocloud.cmp.tencentconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.vpc.v20170312.models.NetworkInterface;
import com.tencentcloudapi.vpc.v20170312.models.NetworkInterfaceAttachment;

/**
 * tencent网卡 转换
 *
 * @author ymh
 * @时间 2018年11月22日
 */
public class NetworkCardConvertor implements BeanConvertor<NetworkCardModel, NetworkInterface> {

    @Override
    public List<NetworkCardModel> convertList(List<? extends NetworkInterface> objects) {
        List<NetworkCardModel> regionModels = new ArrayList<>();
        for (NetworkInterface object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public NetworkCardModel convertModel(NetworkInterface info) {
        NetworkCardModel model = new NetworkCardModel();
        if (null != info) {
            model.setNetworkcardUuid(info.getNetworkInterfaceId());
            model.setName(info.getNetworkInterfaceName());
            model.setRemark(info.getNetworkInterfaceDescription());
            model.setMac(info.getMacAddress());
            model.setIsPrimary(info.getPrimary());
            model.setStatus(info.getState());
            model.setSubnetUuid(info.getSubnetId());
            model.setVpcUuid(info.getVpcId());
            model.setZone(info.getZone());
            // 内网IP
            model.setIpAddress(JSON.toJSONString(info.getPrivateIpAddressSet()));
            NetworkInterfaceAttachment attachment = info.getAttachment();
            model.setUsed(false);
            model.setLabel(null);
            if (null != attachment) {
                String instanceId = attachment.getInstanceId();
                if (null != instanceId) {
                    model.setUsed(true);
                    model.setLabel(instanceId);
                }
                model.setInstanceId(instanceId);
            }
            String createdTime = info.getCreatedTime();
            if (!StringUtils.isEmpty(createdTime)) {
                model.setGmtCreate(DateTools.stringToDateTime(createdTime));
            }
        }
        return model;
    }
}
