package com.bocloud.cmp.tencentconvertor;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.vpc.v20170312.models.Subnet;

/**
 * 地域 转换
 * 
 * @author ymh
 * @时间 2018年11月22日
 */
public class SubnetConvertor implements BeanConvertor<SubnetModel, Subnet> {

    private static Logger logger = LoggerFactory.getLogger(SubnetConvertor.class);

    @Override
    public List<SubnetModel> convertList(List<? extends Subnet> objects) {
        List<SubnetModel> regionModels = new ArrayList<SubnetModel>();
        for (Subnet object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public SubnetModel convertModel(Subnet info) {
        SubnetModel model = new SubnetModel();
        if (null != info) {
            String subnetName = info.getSubnetName();
            model.setName(subnetName);
            model.setSubnetUuid(info.getSubnetId());
            model.setCidr(info.getCidrBlock());
            model.setIpv6Cidr(info.getIpv6CidrBlock());
            model.setZone(info.getZone());
            model.setIpVersion("V4");
            model.setNetworkUuid(info.getVpcId());
            model.setVpcUuid(info.getVpcId());
            model.setZone(info.getZone());
            model.setIsDefault(info.getIsDefault());
            model.setRouteTableId(info.getRouteTableId());
            model.setAvailableIpAddressCount(info.getAvailableIpAddressCount());
            String createdTime = info.getCreatedTime();
            if (null != createdTime && !createdTime.equals("0000-00-00 00:00:00")) {
                model.setGmtCreate(DateTools.stringToDateTime(createdTime));
            }
        }
        return model;
    }
}
